//
//  Controller.m
//  YKExplorer
//
//  Created by KOYAMA Yoshiaki on Thu Dec 25 2003.
//  Copyright (c) 2003 KOYAMA Yoshiaki. All rights reserved.
//

#import "Controller.h"
#import "IconCell.h"
#import "DataSource.h"
#import "FileSystemItem.h"

static NSString *YKExplorerToolbarIdentifier = @"YKExplorer Toolbar";
static NSString *FinderToolbarItemIdentifier = @"Finder ToolbarItem";
static NSString *PathToolbarItemIdentifier = @"Path ToolbarItem";
static NSString *OpenToolbarItemIdentifier = @"Open ToolbarItem";
static NSString *CutFileToolbarItemIdentifier = @"Cut File ToolbarItem";
static NSString *CopyFileToolbarItemIdentifier = @"Copy File ToolbarItem";
static NSString *MoveFileToolbarItemIdentifier = @"Move File ToolbarItem";
static NSString *TrashFileToolbarItemIdentifier = @"Trash File ToolbarItem";
static NSString *TextFileToolbarItemIdentifier = @"Text File ToolbarItem";

@implementation Controller


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
}

// EChE̏
- (BOOL)windowShouldClose:(id)sender {
    [NSApp terminate:self];
    return YES;
}

// Toolbar̍쐬
- (void)awakeFromNib {
	NSFont *aFont = [NSFont fontWithName:@"HiraKakuPro-W3" size:10];
	
	[self initializeToolbar];
	_srcPath = nil;
	[_textView setFont:aFont];
}

// Toolbaȑ
- (void)initializeToolbar {
    NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier:YKExplorerToolbarIdentifier];
	NSFont *aFont;

    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    [toolbar setDisplayMode:NSToolbarDisplayModeIconOnly];
    [toolbar setDelegate:self];
    [_window setToolbar:toolbar];
    [toolbar release];

	aFont = [NSFont fontWithName:@"HiraKakuPro-W3" size:10];
	[_popUp setFont:aFont];

}


- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar
     itemForItemIdentifier:(NSString *)itemIdent
 willBeInsertedIntoToolbar:(BOOL)willBeInserted
{
    NSToolbarItem *toolbarItem = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdent];
	NSString *itemStr;

    [toolbarItem autorelease];
    
	if([itemIdent isEqual:FinderToolbarItemIdentifier]) {
        [toolbarItem setLabel: @"Finder"];
        [toolbarItem setPaletteLabel: @"Finder"];
        [toolbarItem setToolTip:@"Finder"];
        [toolbarItem setImage:[NSImage imageNamed: @"Finder"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(openFolder:)];
    } else if([itemIdent isEqual:OpenToolbarItemIdentifier]) {
		itemStr = NSLocalizedString(@"BAR_OPEN",@"Open");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Application"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(openFile:)];
    } else if([itemIdent isEqual:PathToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_PATH",@"Path");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Path"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(copyPasteboard:)];
    } else if([itemIdent isEqual:CopyFileToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_COPY",@"Copy");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Copy"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(copyFile:)];
    } else if([itemIdent isEqual:MoveFileToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_MOVE",@"Move");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Move"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(moveFile:)];
	} else if([itemIdent isEqual:CutFileToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_CUT",@"Cut");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Cut"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(cutFile:)];
	} else if([itemIdent isEqual:TrashFileToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_TRASH",@"Trash");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"Trash"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(trashFile:)];
	} else if([itemIdent isEqual:TextFileToolbarItemIdentifier]) {		
		itemStr = NSLocalizedString(@"BAR_VIEW",@"View");
		[toolbarItem setLabel: itemStr];
        [toolbarItem setPaletteLabel: itemStr];
        [toolbarItem setToolTip: itemStr];
        [toolbarItem setImage:[NSImage imageNamed: @"View"]]; //32*32
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(viewFile:)];
	} else {
        return nil;
    }
	
    return toolbarItem;
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar
{
    return [NSArray arrayWithObjects:
		FinderToolbarItemIdentifier, 
		NSToolbarSeparatorItemIdentifier,
		OpenToolbarItemIdentifier, PathToolbarItemIdentifier,
		TextFileToolbarItemIdentifier,
		NSToolbarSeparatorItemIdentifier,
		CutFileToolbarItemIdentifier, CopyFileToolbarItemIdentifier,
		MoveFileToolbarItemIdentifier,
		NSToolbarSeparatorItemIdentifier,
		TrashFileToolbarItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier,
        NSToolbarCustomizeToolbarItemIdentifier, nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar
{ 
    return [NSArray arrayWithObjects:
        FinderToolbarItemIdentifier,
        OpenToolbarItemIdentifier, PathToolbarItemIdentifier,
		CutFileToolbarItemIdentifier, CopyFileToolbarItemIdentifier,
		MoveFileToolbarItemIdentifier, TrashFileToolbarItemIdentifier,
		TextFileToolbarItemIdentifier,
//		NSToolbarPrintItemIdentifier, NSToolbarShowColorsItemIdentifier,
//		NSToolbarShowFontsItemIdentifier, 
//		NSToolbarCustomizeToolbarItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier, NSToolbarSpaceItemIdentifier,
        NSToolbarSeparatorItemIdentifier, nil];
}

- (void)toolbarWillAddItem:(NSNotification *)notification
{ 
    NSToolbarItem *addedItem = [[notification userInfo] objectForKey: @"item"];
    if ([[addedItem itemIdentifier] isEqual:FinderToolbarItemIdentifier]) {
    }
}

- (void)toolbarDidRemoveItem:(NSNotification *)notification
{ 
/*
    NSToolbarItem *removedItem = [[notification userInfo] objectForKey: @"item"];
   if (removedItem == item) {
    }
*/
}

// Toolbar̗LE̔f
- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem
{
    BOOL ret = YES;
	NSString *path;
	int row;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	BOOL isDirPath;
	NSString *aPath = nil;
	NSMutableArray *array;
	BOOL bundle;

	if (([[toolbarItem itemIdentifier] isEqual:OpenToolbarItemIdentifier]) ||
		([[toolbarItem itemIdentifier] isEqual:PathToolbarItemIdentifier])) {
		row = [_tableView selectedRow];
		if (row == -1) {
			ret = NO;
		}
	} else if ([[toolbarItem itemIdentifier] isEqual:FinderToolbarItemIdentifier]) {
		path = [_dataSource outlineViewPath];
		if (path == nil) {
			ret = NO;
		}
	} else if (([[toolbarItem itemIdentifier] isEqual:CopyFileToolbarItemIdentifier]) ||
			   ([[toolbarItem itemIdentifier] isEqual:MoveFileToolbarItemIdentifier])) {
		path = [_dataSource outlineViewPath];
		if ((path == nil) || (_srcPath == nil)) {
			ret = NO;
		}
	} else if (([[toolbarItem itemIdentifier] isEqual:CutFileToolbarItemIdentifier]) || 
			   ([[toolbarItem itemIdentifier] isEqual:TrashFileToolbarItemIdentifier]) ||
			   ([[toolbarItem itemIdentifier] isEqual:TextFileToolbarItemIdentifier])) {
		isDirPath = NO;
		bundle = NO;
		row = [_tableView selectedRow];
		array = [_dataSource tableViewArray];

		if (row != -1) {
			aPath = [array objectAtIndex:row];
			bundle = checkBundle(aPath);
			[fileManager fileExistsAtPath:aPath isDirectory:&isDirPath];
		}

		if ((row == -1) || ((isDirPath == YES) && (bundle == NO))) {
			ret = NO;
		}
	}
		
	return ret;
}

// t@CAtH_J
- (void)openFile:(id)sender {
	NSWorkspace *wspace = [NSWorkspace sharedWorkspace];
	NSMutableArray *array;
	NSString *path;
	int row;

	row = [_tableView selectedRow];
	if (row != -1) {
		array = [_dataSource tableViewArray];
		path = [array objectAtIndex:row];
		[wspace openFile:path];
	}
}

// tH_[FinderŊJ
- (void)openFolder:(id)sender {
	NSWorkspace *wspace = [NSWorkspace sharedWorkspace];
	NSString *path;

	path = [_dataSource outlineViewPath];
	if (path != nil) {
		[wspace openFile:path];
	}
}

// t@C̃pXNbv{[hɃRs[
- (void)copyPasteboard:(id)sender {
	NSMutableArray *array;
	NSString *path;
	int row;
	NSArray *types = [NSArray arrayWithObject:NSStringPboardType];
	NSPasteboard *pb = [NSPasteboard pasteboardWithName:NSGeneralPboard];
	NSString *format, *message;

	row = [_tableView selectedRow];
	if (row != -1) {
		BOOL success;

		array = [_dataSource tableViewArray];
		path = [array objectAtIndex:row];
		[pb declareTypes:types owner:self];
		success = [pb setString:path forType:NSStringPboardType];

		if (success == YES) {
			format = NSLocalizedString(@"MESSAGE_PATHS",@"");
			message = [NSString stringWithFormat:format, [path lastPathComponent]];
		} else {
			format = NSLocalizedString(@"MESSAGE_PATHE",@"");
			message = [NSString stringWithFormat:format, [path lastPathComponent]];
		}
		[_label setStringValue:message];
	}

}

// ړݒ
- (void)cutFile:(id)sender {
	NSString *path;
	NSMutableArray *array;
	int row;
	NSString *message, *format;

	row = [_tableView selectedRow];
	if (row != -1) {

		array = [_dataSource tableViewArray];
		path = [array objectAtIndex:row];
		_srcPath = [path retain];
		format = NSLocalizedString(@"MESSAGE_CUT",@"");

		message = [NSString stringWithFormat:format, [_srcPath lastPathComponent]];
		[_label setStringValue:message];
//		NSLog(@"%@", _srcPath);
	}
}

// t@C̃Rs[
- (void)copyFile:(id)sender {
	NSWorkspace *wspace = [NSWorkspace sharedWorkspace];
	FileSystemItem *rootItem;
	NSString *format, *message;
	NSString *source, *dest, *file;
	BOOL flag;
	int aTag;
	NSArray *files;
	
	dest = [_dataSource outlineViewPath];
	if (dest != nil) {

		source = [_srcPath stringByDeletingLastPathComponent];
		file = [_srcPath lastPathComponent];
		files = [NSArray arrayWithObject:file];
			
		// t@C̃Rs[
		flag = [wspace performFileOperation:NSWorkspaceCopyOperation source:source destination:dest files:files tag:&aTag];

		if (flag == YES) {
			format = NSLocalizedString(@"MESSAGE_COPYS",@"");
			message = [NSString stringWithFormat:format, [_srcPath lastPathComponent]];
		} else {
			format = NSLocalizedString(@"MESSAGE_COPYE",@"");
			message = [NSString stringWithFormat:format, [_srcPath lastPathComponent]];
		}
		[_label setStringValue:message];

		rootItem = [FileSystemItem rootItem];
//		[_outlineView reloadItem:rootItem reloadChildren:YES];
		
		[_dataSource setPathTableView:dest];
		[_tableView reloadData];
	}
}

// t@C̈ړ
- (void)moveFile:(id)sender {
	NSWorkspace *wspace = [NSWorkspace sharedWorkspace];
	FileSystemItem *rootItem;
	NSString *format, *message;
	BOOL flag;
	int aTag;
	NSString *source, *dest, *file;
	NSArray *files;

	dest = [_dataSource outlineViewPath];
	if (dest != nil) {

		source = [_srcPath stringByDeletingLastPathComponent];
		file = [_srcPath lastPathComponent];
		files = [NSArray arrayWithObject:file];
			
		// t@C̈ړ
		flag = [wspace performFileOperation:NSWorkspaceMoveOperation source:source destination:dest files:files tag:&aTag];

		if (flag == YES) {
			format = NSLocalizedString(@"MESSAGE_MOVES",@"");
			message = [NSString stringWithFormat:format, [_srcPath lastPathComponent]];
		} else {
			format = NSLocalizedString(@"MESSAGE_MOVEE",@"");
			message = [NSString stringWithFormat:format, [_srcPath lastPathComponent]];
		}
		[_label setStringValue:message];

		rootItem = [FileSystemItem rootItem];
//		[_outlineView reloadItem:rootItem reloadChildren:YES];
		
		[_dataSource setPathTableView:dest];
		[_tableView reloadData];
	}
}

// t@CS~Ɉړ
- (void)trashFile:(id)sender {
	NSWorkspace *wspace = [NSWorkspace sharedWorkspace];
	NSMutableArray *array;
	NSString *path;
	int row;
	BOOL flag;
	int aTag;
	NSString *source, *file;
	NSArray *files;
	NSString *format, *message;
	FileSystemItem *rootItem;
	NSString *pathOutline;

	row = [_tableView selectedRow];
	if (row != -1) {

		array = [_dataSource tableViewArray];
		path = [array objectAtIndex:row];

		source = [path stringByDeletingLastPathComponent];
		file = [path lastPathComponent];	
		files = [NSArray arrayWithObject:file];
		
		// t@C̈ړ
		flag = [wspace performFileOperation:NSWorkspaceRecycleOperation source:source destination:source files:files tag:&aTag];

		if (flag == YES) {
			format = NSLocalizedString(@"MESSAGE_TRASHS",@"");
			message = [NSString stringWithFormat:format, [path lastPathComponent]];
		} else {
			format = NSLocalizedString(@"MESSAGE_TRASHE",@"");
			message = [NSString stringWithFormat:format, [path lastPathComponent]];
		}
		[_label setStringValue:message];
		
		rootItem = [FileSystemItem rootItem];
//		[_outlineView reloadItem:rootItem reloadChildren:YES];
		
		pathOutline = [_dataSource outlineViewPath];
		[_dataSource setPathTableView:pathOutline];
		[_tableView reloadData];

	}
}

// eLXg̕\
- (void)viewFile:(id)sender {
	NSMutableArray *array;
	NSString *path;
	NSString *text = nil;
	int row;
	NSString *tab;
	NSString *ext;
	BOOL flag;
	NSString *format, *message;
	NSFont *aFont = [NSFont fontWithName:@"HiraKakuPro-W3" size:10];

	tab = [[_tabView selectedTabViewItem] identifier];
	if ([tab isEqualToString:@"1"] == YES) {
		[_textView setString:@""];
		[_textView setFont:aFont];
		[_popUp selectItemAtIndex:0];
		[_tabView selectTabViewItemWithIdentifier:@"2"];
		
		row = [_tableView selectedRow];
		array = [_dataSource tableViewArray];
		path = [array objectAtIndex:row];

		ext = [path pathExtension];
		if (([ext caseInsensitiveCompare:@"rtf"] == NSOrderedSame) ||
			([ext caseInsensitiveCompare:@"rtfd"] == NSOrderedSame)) {
			flag = [_textView readRTFDFromFile:path];

				if (flag == NO) {
					format = NSLocalizedString(@"MESSAGE_VIEWE",@"");
					message = [NSString stringWithFormat:format, [path lastPathComponent]];
				} else {
					format = NSLocalizedString(@"MESSAGE_VIEWS",@"");
					message = [NSString stringWithFormat:format, [path lastPathComponent]];
				}
				[_labelText setStringValue:message];
		} else {

			text = [NSString stringWithContentsOfFile:path];
			if (text == nil) {
				format = NSLocalizedString(@"MESSAGE_VIEWE",@"");
				message = [NSString stringWithFormat:format, [path lastPathComponent]];
			} else {
				format = NSLocalizedString(@"MESSAGE_VIEWS",@"");
				message = [NSString stringWithFormat:format, [path lastPathComponent]];
				[_textView setString:text];
			}
			[_labelText setStringValue:message];
		}
	} else {
		[_tabView selectTabViewItemWithIdentifier:@"1"];
	}
}

-(IBAction)selectPopUp:(id)sender {
	NSMutableArray *array;
	NSString *path;
	NSString *text = nil;
	NSData *data;
	NSString *message, *format;
	int row;
	int tag;
	NSFont *aFont = [NSFont fontWithName:@"HiraKakuPro-W3" size:10];

	[_textView setString:@""];
	[_textView setFont:aFont];

	row = [_tableView selectedRow];
	array = [_dataSource tableViewArray];
	path = [array objectAtIndex:row];

	
	tag = [[sender selectedItem] tag];
	data = [NSData dataWithContentsOfFile:path];

	switch (tag) {
	case 1:
		text = [NSString stringWithContentsOfFile:path];
		break;
	case 2:
		// Shift-JIS
		text = [[NSString alloc] initWithData:data encoding: NSShiftJISStringEncoding];
		break;
	case 3:
		// JIS
		text = [[NSString alloc] initWithData:data encoding: NSISO2022JPStringEncoding];
		break;
	case 4:
		// EUC
		text = [[NSString alloc] initWithData:data encoding: NSJapaneseEUCStringEncoding];
		break;
	case 5:
		// UTF-8
		text = [[NSString alloc] initWithData:data encoding: NSUTF8StringEncoding];
		break;
	case 6:
		// UTF-16
		text = [[NSString alloc] initWithData:data encoding: NSUnicodeStringEncoding];
		break;
	}

	if (text == nil) {
		[_textView setString:@""];
		format = NSLocalizedString(@"MESSAGE_VIEWE",@"");
		message = [NSString stringWithFormat:format, [path lastPathComponent]];
	} else {
		format = NSLocalizedString(@"MESSAGE_VIEWS",@"");
		message = [NSString stringWithFormat:format, [path lastPathComponent]];
		[_textView setString:text];
	}
	[_labelText setStringValue:message];
}

@end
